from __future__ import print_function, absolute_import
from __future__ import unicode_literals
import argparse
import logging
import logging.config
import sys

from .helpers import file_or_resource
from .validator import validate
from opentargets_urlzsource import URLZSource


def main():
    logging.config.fileConfig(file_or_resource('logging.ini'),
                              disable_existing_loggers=False)
    logger = logging.getLogger(__name__)

    parser = argparse.ArgumentParser(description='OpenTargets evs validator')
    parser.add_argument('data_source_file', nargs='?', default='-',
                        help='The prefix to prepend default: STDIN')
    parser.add_argument("--schema", dest='schema',
                        help="set the schema file to use",
                        action='store')
    parser.add_argument("--log-level", dest='loglevel',
                        help="set the log level def: WARNING",
                        action='store', default='WARNING')
    parser.add_argument("--log-lines", dest='loglines',
                        help="number of log errors to print out [no longer supported]",
                        action='store', type=int, default = None)
    parser.add_argument("--hash", dest='hash',
                        help="calculate hash of each line for uniqueness",
                        action='store_true')

    args = parser.parse_args()

    if args.loglevel:
        try:
            root_logger = logging.getLogger()
            root_logger.setLevel(logging.getLevelName(args.loglevel))
            logger.setLevel(logging.getLevelName(args.loglevel))
        except Exception as e:
            root_logger.exception(e)

    #TODO use a position argument
    if not args.schema:
        logger.error('A --schema <schemafile> has to be specified.')
        return 1

    # warn and exit when using removed arguments
    if args.loglines is not None:
        logger.error("--log-lines is no longer supported")
        return 3


    valid = True
    if args.data_source_file == '-':
        valid = validate(sys.stdin, args.schema, args.hash)
    else:
        with URLZSource(args.data_source_file).open() as fh:
            valid = validate(fh, args.schema, args.hash)

    #if we had any validation errors, exit with status 2
    if not valid:
        return 2

    #if everything was fine, exit with status 0
    return 0


if __name__ == '__main__':
    sys.exit(main())



from __future__ import unicode_literals
from builtins import object
import logging
import requests
import jsonschema as jss
import pkg_resources as res
import simplejson as json
import os
import rfc3987
import opentargets_validator
from collections import OrderedDict
import hashlib


def file_handler(uri):
    #handle file:// uris because of https://github.com/Julian/jsonschema/issues/478

    schema = None
    uri_split = rfc3987.parse(uri)
    with open(os.path.abspath(os.path.join(uri_split['authority'], uri_split['path'])), 'r') as schema_file:
        schema = json.load(schema_file)
    return schema


def generate_validator_from_schema(schema_uri):

    #download the schema to a string
    schema = None
    #handle http and file
    uri_split = rfc3987.parse(schema_uri)
    if uri_split['scheme'] in ("http", "https"):
        #its a http or https use requests
        schema = requests.get(schema_uri).json()
    elif uri_split['scheme'] == "file":
        #its a file, open as normal
        #reconstiture the file path from the uri
        with open(os.path.abspath(os.path.join(uri_split['authority'], uri_split['path'])), 'r') as schema_file:
            schema = json.load(schema_file)
    else:
        raise ValueError("schema uri must have file or url scheme")
    

    #Create a refresolver to allow resolution
    #of relative schema links
    #This is required to use git branches / versions and
    #local development correctly
    #Don't use from_schema because it uses the $id baked
    #into the schema, and we want to avoid baking
    handlers = dict(file=file_handler)
    resolver = jss.RefResolver(schema_uri, schema, handlers=handlers, 
        store={})

    validator = jss.Draft7Validator(schema=schema, 
        resolver=resolver,
        )

    return validator

def file_or_resource(fname=None):
    '''get filename and check if in getcwd then get from
    the package resources folder
    '''
    if fname is not None:
        filename = os.path.expanduser(fname)
    
        resource_package = opentargets_validator.__name__
        resource_path = os.path.sep.join(('resources', filename))

        abs_filename = os.path.join(os.path.abspath(os.getcwd()), filename) \
                       if not os.path.isabs(filename) else filename

        return abs_filename if os.path.isfile(abs_filename) \
            else res.resource_filename(resource_package, resource_path)




#!/usr/bin/env python

from __future__ import unicode_literals
from setuptools import setup
import io
import os


pkg_dir = os.path.dirname(__file__)
# importing __<vars>__ into the namespace
#https://packaging.python.org/guides/single-sourcing-package-version/#single-sourcing-the-version
with open(os.path.join(pkg_dir, 'opentargets_validator', 'version.py')) as fv:
    exec(fv.read())


long_description = ""
readme_path = os.path.join(pkg_dir, 'README.md')
with io.open(readme_path, encoding='utf-8') as readme_file:
    long_description = readme_file.read()


setup(name=__pkgname__,
    version=__version__,
    description=__description__,
    long_description=long_description,
    long_description_content_type='text/markdown; charset=UTF-8',
    author=__author__,
    author_email=__author_email__,
    url=__homepage__,
    packages=['opentargets_validator'],
    license=__license__,
    download_url=__homepage__ + '/archive/' + __version__ + '.tar.gz',
    keywords=['opentargets', 'bioinformatics', 'python2'],
    platforms=['any'],
    #make sure this matches requirements.txt
    install_requires=['requests','jsonschema==3.0.0a3', 'rfc3987', 'future', 'simplejson', 'pypeln==0.1.6', 'opentargets-urlzsource'],
    dependency_links=[],
    include_package_data=True,
    entry_points={'console_scripts': ['opentargets_validator=opentargets_validator.cli:main'],},
    data_files=[],
    scripts=[],
    classifiers=[
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 3'
    ],
    #make sure this matches requirements.txt
    extras_require={'dev': ['pytest-cov','codecov']}
)



from __future__ import absolute_import
from __future__ import unicode_literals
from builtins import str
import logging
import simplejson as json
import multiprocessing
import hashlib
from .helpers import generate_validator_from_schema
import pypeln
import functools


def validate_start(schema_uri):
    validator = generate_validator_from_schema(schema_uri)
    logger = logging.getLogger(__name__)
    return validator, logger

def validator_mapped(data, validator, logger):
    line_counter, line = data
    
    try:
        parsed_line = json.loads(line)
    except Exception as e:
        logger.error('failed parsing line %i: %s', line_counter, e)
        return line_counter, None, None

    validation_errors = [(".".join(error.absolute_path), error.message) for error in validator.iter_errors(parsed_line)]

    hash_line = hashlib.md5(json.dumps(parsed_line["unique_association_fields"], 
        sort_keys=True).encode("utf-8")).hexdigest()

    return line_counter, validation_errors, hash_line

def validate(file_descriptor, schema_uri, do_hash):
    logger = logging.getLogger(__name__)
    hash_lines = dict()
    input_valid = True

    cpus = multiprocessing.cpu_count()

    stage = pypeln.process.map(validator_mapped, enumerate(file_descriptor, start=1),
        on_start=functools.partial(validate_start, schema_uri),
        workers=cpus,
        maxsize=1000)

    for line_counter, validation_errors, hash_line in stage:
        line_valid = True

        if validation_errors:
            line_valid = False
            input_valid = False
            for path, message in validation_errors:
                logger.error('fail @ %i.%s %s', line_counter, path, message)


        #check for any hash collisions
        #only check those that have passed validation so far
        if do_hash and line_valid:
            if hash_line in hash_lines:
                #duplicate hash, fail this line
                line_valid = False
                input_valid = False

                # order the lies so log is sensible
                # might not be ordered due to pypeln multiprocessing
                line_min = min(line_counter, hash_lines[hash_line])
                line_max = max(line_counter, hash_lines[hash_line])
                logger.error("Duplicate hashes %d and %d ",
                    line_min, line_max)
            else:
                hash_lines[hash_line] = line_counter

        line_counter += 1

    #check if we had no lines, if so something went wrong and needs to be flagged
    if line_counter == 0:
        logger.error("No lines in input - does it exist?")
        input_valid = False

    return input_valid
